#! /bin/sh

cleanup () { 
  erl -setcookie $$ -sname cleanup$$ -noshell -noinput -eval '
    Host = lists:last (string:tokens (atom_to_list (node ()), "@")),
    [ One, Two ] = init:get_plain_arguments (),
    rpc:call (list_to_atom (One ++ "@" ++ Host), erlang, halt, []),
    rpc:call (list_to_atom (Two ++ "@" ++ Host), erlang, halt, [])
  ' -s erlang halt -extra one$$ two$$

  rm -rf Mnesia.one$$*
  rm -rf Mnesia.two$$*
}

trap 'exit 1;' INT HUP TERM QUIT
trap 'cleanup;' EXIT

rm -rf Mnesia.one$$*
rm -rf Mnesia.two$$*

erl -setcookie $$ -sname one$$ -noshell -noinput -eval '
  ok = mnesia:start (),
  { atomic, ok } = mnesia:change_table_copy_type (schema, node (), disc_copies),
  receive after infinity -> ok end
' -s erlang halt >/dev/null 2>/dev/null &

sleep 2

erl -setcookie $$ -sname two$$ -noshell -noinput -eval '
  Host = lists:last (string:tokens (atom_to_list (node ()), "@")),
  [ One ] = init:get_plain_arguments (),
  Other = list_to_atom (One ++ "@" ++ Host),
  pong = net_adm:ping (Other),
  ok = mnesia:start (),
  { ok, _ } = mnesia:change_config (extra_db_nodes, [ Other ]),
  { atomic, ok } = mnesia:change_table_copy_type (schema, node (), disc_copies),
  { atomic, ok } = 
    mnesia:create_table (flass, 
                         [ { frag_properties, [
                               { n_disc_copies, 2 }
                           ] } ]),
  { atomic, ok } = 
    mnesia:create_table (turg, 
                         [ { frag_properties, [
                               { foreign_key, { flass, key } }
                           ] } ]),
  { value, { node_pool, TurgPool } } = lists:keysearch (node_pool, 1, mnesia:table_info (turg, frag_properties)),
  [] = TurgPool -- [ node (), Other ],
  { value, { node_pool, FlassPool } } = lists:keysearch (node_pool, 1, mnesia:table_info (flass, frag_properties)),
  [] = FlassPool -- [ node (), Other ],
  stopped = rpc:call (Other, mnesia, stop, []),
  { atomic, ok } = mnesia:del_table_copy (schema, Other),
  { value, { node_pool, TurgPoolTwo } } = lists:keysearch (node_pool, 1, mnesia:table_info (turg, frag_properties)),
  [] = TurgPoolTwo -- [ node () ],
  { value, { node_pool, FlassPoolTwo } } = lists:keysearch (node_pool, 1, mnesia:table_info (flass, frag_properties)),
  case FlassPoolTwo -- [ node () ] of
    [] -> io:format ("PASS: table referred to by foreign key has proper node pool.~n", []);
    _ -> 
      io:format ("FAIL: table referred to by foreign key has removed nodes in node pool.~n", []),
      io:format ("trying manual removal...~n", []),
      io:format ("~p~n", [ mnesia:change_table_frag (flass, { del_node, Other }) ]),
      halt (1)
  end
' -s erlang halt -extra one$$
